/*
 * iocsh asynOctet device support
 * 
 * Author: W. Eric Norum
 * "2011/02/22 16:18:28 (UTC)"
 */

/************************************************************************\
* Copyright (c) 2011 Lawrence Berkeley National Laboratory, Accelerator
* Technology Group, Engineering Division
* This code is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

#include <ctype.h>
#include <cantProceed.h>
#include <epicsStdio.h>
#include <asynDriver.h>
#include <asynOctet.h>
#include <iocsh.h>
#include <epicsExport.h>


/*
 * Private storage
 */
typedef struct iocshPvt {
    asynInterface  asynCommon;     /* Our interfaces */
    asynInterface  asynOctet;
} iocshPvt;

/*
 * asynCommon methods
 */
static void
report(void *pvt, FILE *fp, int details)
{
}

static asynStatus
connect(void *pvt, asynUser *pasynUser)
{
    return pasynManager->exceptionConnect(pasynUser);
}

static asynStatus
disconnect(void *pvt, asynUser *pasynUser)
{
    return pasynManager->exceptionDisconnect(pasynUser);
}
static asynCommon commonMethods = { report, connect, disconnect };

/*
 * asynOctet methods
 */
static asynStatus
octetWrite(void *pvt, asynUser *pasynUser, const char *data, size_t numchars, size_t *nTrans)
{

    const char *cp;
    size_t n;

    *nTrans = 0;
    if ((numchars < 2) || (data[numchars-1] != '\0')) {
        epicsSnprintf(pasynUser->errorMessage, pasynUser->errorMessageSize,
                                      "Value is not a null-terminated string");
        return asynError;
    }
    for (cp = data, n = 0 ; n < numchars ; n++, cp++) {
        if (!isascii((unsigned char)*cp)) {
            epicsSnprintf(pasynUser->errorMessage, pasynUser->errorMessageSize,
                                      "Value is contains non-ASCII characters");
            return asynError;
        }
    }
    *nTrans = numchars;
    iocshCmd(data);
    return asynSuccess;
}

static asynOctet octetMethods = { octetWrite };


static void
iocshDevRegister(void)
{
    iocshPvt *ppvt;
    char *portName = "IOCSH";
    asynStatus status;
    static int firstTime = 1;

    /*
     * Only do this once
     */
    if (!firstTime)
        return;
    firstTime = 0;

    /*
     * Create private data area
     */
    ppvt = callocMustSucceed(1, sizeof(iocshPvt), "iocshDevRegister");

    /*
     * Create port
     */
    status = pasynManager->registerPort(portName,
                                        ASYN_CANBLOCK,
                                        1,  /*  autoconnect */
                                        0,  /* medium priority (for now) */
                                        0); /* default stack size */
    if (status != asynSuccess) {
        printf("Can't register port %s", portName);
        return;
    }

    /*
     * Advertise our asynCommon interface
     */
    ppvt->asynCommon.interfaceType = asynCommonType;
    ppvt->asynCommon.pinterface  = &commonMethods;
    ppvt->asynCommon.drvPvt = ppvt;
    status = pasynManager->registerInterface(portName, &ppvt->asynCommon);
    if (status != asynSuccess) {
        printf("Can't register asynCommon support.\n");
        return;
    }

    /*
     * Advertise our asynIntOctet interface
     */
    ppvt->asynOctet.interfaceType = asynOctetType;
    ppvt->asynOctet.pinterface = &octetMethods;
    ppvt->asynOctet.drvPvt = ppvt;
    status = pasynManager->registerInterface(portName, &ppvt->asynOctet);
    if (status != asynSuccess) {
        printf("Can't register asynOctet support.\n");
        return;
    }
}


epicsExportRegistrar(iocshDevRegister);

